/******************************************************************************
*                                                                             *
*       Copyright (C) 1993 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <stdio.h>

/* These routines allocate memory which may be deallocaed with free().
   There are seven basic memory sizes, char (c), short (s), int (i),
   long (l), float (f), double (d) and pointer (p).  These may be
   preceeded by u for unsigned in the case of integer values.

   vector gives a one dimensional array, matrix a two dimensional array
   and tensorx a x dimensional array.

   All routines call Smalloc to perform error checking */

char *Svectorc(int nitem) {
  return((char*) Smalloc((unsigned) (nitem * sizeof(char))));
}

unsigned char Svectoruc(int nitem) {
  return((unsigned char*) Smalloc((unsigned) (nitem * sizeof(unsigned char))));
}

short *Svectors(int nitem) {
  return((short*) malloc((unsigned) (nitem * sizeof(short))));
}

unsigned short *Svectorus(int nitem) {
  return((unsigned short*) malloc((unsigned) (nitem *sizeof(unsigned short))));
}

int *Svectori(int nitem) {
  return((int*) malloc((unsigned) (nitem * sizeof(int))));
}

unsigned int *Svectorui(int nitem) {
  return((unsigned int*) malloc((unsigned) (nitem * sizeof(unsigned int))));
}

long *Svectorl(int nitem) {
  return((long*) malloc((unsigned) (nitem * sizeof(long))));
}

unsigned long *Svectorul(int nitem) {
  return((unsigned long*) malloc((unsigned) (nitem * sizeof(unsigned long))));
}

float *Svectorf(int nitem) {
  return((float*) malloc((unsigned) (nitem * sizeof(float))));
}

double *Svectord(int nitem) {
  return((double*) malloc((unsigned) (nitem * sizeof(double))));
}

void **Svectorp(int nitem) {
  return((void**) malloc((unsigned) (nitem * sizeof(void*))));
}

/*****************************/
/* start of matrix functions */

char **Smatrixc(int nitem0, int nitem1) {
  char **array0 = (char**) Smalloc(nitem0 * sizeof(char*) + 
				   nitem0 * nitem1 * sizeof(char));
  char  *array1 = (char*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

unsigned char **Smatrixuc(int nitem0, int nitem1) {
  unsigned char **array0 = (unsigned char**)
    Smalloc(nitem0 * sizeof(unsigned char*) +
	    nitem0 * nitem1 * sizeof(unsigned char));
  unsigned char  *array1 = (unsigned char*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

short **Smatrixs(int nitem0, int nitem1) {
  short **array0 = (short**) Smalloc(nitem0 * sizeof(short*) + 
				     nitem0 * nitem1 * sizeof(short));
  short  *array1 = (short*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

unsigned short **Smatrixus(int nitem0, int nitem1) {
  unsigned short **array0 = (unsigned short**)
    Smalloc(nitem0 * sizeof(unsigned short*) +
	    nitem0 * nitem1 * sizeof(unsigned short));
  unsigned short  *array1 = (unsigned short*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

int **Smatrixi(int nitem0, int nitem1) {
  int **array0 = (int**) Smalloc(nitem0 * sizeof(int*) + 
				 nitem0 * nitem1 * sizeof(int));
  int  *array1 = (int*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

unsigned int **Smatrixui(int nitem0, int nitem1) {
  unsigned int **array0 = (unsigned int**)
    Smalloc(nitem0 * sizeof(unsigned int*) +
	    nitem0 * nitem1 * sizeof(unsigned int));
  unsigned int  *array1 = (unsigned int*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

long **Smatrixl(int nitem0, int nitem1) {
  long **array0 = (long**) Smalloc(nitem0 * sizeof(long*) + 
				   nitem0 * nitem1 * sizeof(long));
  long  *array1 = (long*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

unsigned long **Smatrixul(int nitem0, int nitem1) {
  unsigned long **array0 = (unsigned long**)
    Smalloc(nitem0 * sizeof(unsigned long*) +
	    nitem0 * nitem1 * sizeof(unsigned long));
  unsigned long  *array1 = (unsigned long*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

float **Smatrixf(int nitem0, int nitem1) {
  float **array0 = (float**) Smalloc(nitem0 * sizeof(float*) + 
				     nitem0 * nitem1 * sizeof(float));
  float  *array1 = (float*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

double **Smatrixd(int nitem0, int nitem1) {
  double **array0 = (double**) Smalloc(nitem0 * sizeof(double*) + 
				       nitem0 * nitem1 * sizeof(double));
  double  *array1 = (double*) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}

void ***Smatrixp(int nitem0, int nitem1) {
  void ***array0 = (void***) Smalloc(nitem0 * sizeof(void**) + 
				     nitem0 * nitem1 * sizeof(void*));
  void  **array1 = (void**) (array0 + nitem0);
  int   i;
    
  for(i = 0; i < nitem0; i++)
    array0[i] = array1 + i * nitem1;

  return(array0);
}


char ***Char_3d_array(int nitem0, int nitem1, int nitem2) {
  char ***array0;

  if((array0 = (char***) malloc(nitem0 * sizeof(char**) + 
				nitem0 * nitem1 * sizeof(char*) +
				nitem0 * nitem1 * nitem2)) != NULL) {
    char **array1 = (char**) (array0 + nitem0);
    char  *array2 = (char*)  (array1 + nitem0 * nitem1);
    int  i, j;

    for(i = 0; i < nitem0; i++) {
      array0[i] = array1 + i * nitem1;
      for(j = 0; j < nitem1; j++)
	array0[i][j] = array2 + (i * nitem1 + j) * nitem2;
    }
  }
  
  return(array0);
}

/* general N dimensional routines to go in here one day */
