/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <sys/types.h>
# include <stdio.h>
# include <math.h>
# include <stdlib.h>
# include "Slib.h"

# define DEFAULT_HEADERSIZE	1024
# define DEFAULT_FRAMESTEP	256
# define DEFAULT_WINDOWSIZE	(2 * DEFAULT_FRAMESTEP)
# define DEFAULT_MINSILENCE	0.05
# define DEFAULT_NCEPSTRA	4
# define DEFAULT_MAXDEV		3.0

static int floatcompare(float *a, float *b) {
  return(*a > *b ? 1 : -1);
}

int main(argc, argv) int argc; char **argv; {
  int swapbytes;
  int headersize   = DEFAULT_HEADERSIZE;
  int framestep    = DEFAULT_FRAMESTEP;
  int windowsize   = DEFAULT_WINDOWSIZE;
  float minsilence = DEFAULT_MINSILENCE;
  int ncepstra     = DEFAULT_NCEPSTRA;
  float maxdev	   = DEFAULT_MAXDEV;
  FILE *filei = NULL;
  short *wave;
  float *window, *sortenergy;
  float elimit, **cepCache;
  float *silMean, *silVar, *sigMean, *sigVar;
  int   nsample, frame, nframe;
  int	i, nsil, nsig, nacoustic, initframe, quitframe;

  Sgetargs(argc, argv, "-h", "-help", NULL,
	"-swapbytes",	NULL, &swapbytes, "swap the byte order of the signals",
	"-headersize",	"%d", &headersize, "size of header in bytes",
	"-framestep",	"%d", &framestep, "analysis frame step",
	"-windowsize",	"%d", &windowsize, "analysis window size",
	"-minsilence","%5.3f",&minsilence,"minimum fraction of silence frames",
	"-ncepstra",  "%d",   &ncepstra, "number of cepstral coefficients",
	"-maxdev",   "%5.3f",&maxdev,"maximum deviation of speech from silece",
	NULL,
	"speech file", "r", &filei, NULL, "16 bit speech waveform file",
	NULL);

  /* read in the waveform */
  wave = Sfreadwave(filei, NULL, headersize, windowsize - framestep, &nsample);
  if(swapbytes)
    Sswab(wave, wave, 2 * (nsample + windowsize - framestep));

  nframe = nsample / framestep;
  window = Svector(windowsize, sizeof(*window));
  sortenergy = Svector(nframe, sizeof(*sortenergy));
  cepCache = (float**) Smatrix(nframe, ncepstra, sizeof(**cepCache));

  for(frame = 0; frame < nframe; frame++) {
    SzeroMeanHammWindow(wave + frame * framestep, windowsize, window);
    Swav2cep(window, windowsize, cepCache[frame], ncepstra);
    sortenergy[frame] = cepCache[frame][0];
  }

  qsort((void*) sortenergy, nframe, sizeof(*sortenergy), floatcompare);

  for(frame = 0; frame < nframe; frame++)
    printf("%d\t%f\n", frame, sortenergy[frame]);

  elimit = sortenergy[(int) (minsilence * nframe)];

  /* grab space for acoustic means and variances */
  nacoustic = 2 * ncepstra;
  silMean = Svector(nacoustic, sizeof(*silMean));
  silVar  = Svector(nacoustic, sizeof(*silVar));

  sigMean = Svector(nacoustic, sizeof(*sigMean));
  sigVar  = Svector(nacoustic, sizeof(*sigVar));

  for(i = 0; i < nacoustic; i++) {
    silMean[i] = 0.0;
    silVar[i]  = 0.0;
    sigMean[i] = 0.0;
    sigVar[i]  = 0.0;
  }

  /* make a second pass computing cepstra and delta cepstra sums */
  /* assume a diagional covaariance matrix */
  /* ( forget the first and last frame ) */
  nsil = 0;
  nsig = 0;
  for(frame = 1; frame < nframe - 1; frame++) {
    float **pMean, **pVar;

    if(cepCache[frame][0] < elimit) {
      pMean = &silMean;
      pVar  = &silVar;
      nsil++;
    }
    else {
      pMean = &sigMean;
      pVar  = &sigVar;
      nsig++;
    }
      
    for(i = 0; i < ncepstra; i++) {
      float val = cepCache[frame][i];
      (*pMean)[i] += val;
      (*pVar)[i]  += val * val;
    }
    for(i = 0; i < ncepstra; i++) {
      float val = cepCache[frame - 1][i] - cepCache[frame + 1][i];
      (*pMean)[i + ncepstra] += val;
      (*pVar)[i + ncepstra]  += val * val;
    }
  }	

  /* compute means and variances */
  for(i = 0; i < nacoustic; i++) {
    silMean[i] = silMean[i] / nsil;
    silVar[i]  = silVar[i] / nsil - silMean[i] * silMean[i];

    sigMean[i] = sigMean[i] / nsig;
    sigVar[i]  = sigVar[i] / nsig - sigMean[i] * sigMean[i];

    printf("%d\t%f\t%f\t%f\t%f\n", i, silMean[i], silVar[i],
	   sigMean[i], sigVar[i]);
  }

  {
    int done = 0;

    for(frame = 1; !done && frame < nframe - 1; frame++) {
      int offset;
      float sumvar = 0.0;
      
      for(i = 0; i < ncepstra; i++) {
	float val = cepCache[frame][i] - silMean[i] ;
	sumvar += val * val / silVar[i];
      }
      for(i = 0; i < ncepstra; i++) {
	float val = cepCache[frame - 1][i] - cepCache[frame + 1][i] -
	  silMean[i + ncepstra];
	sumvar += val * val / silVar[i + ncepstra];
      }
/*
      printf("%d\t%f\t%f\n", frame * framestep, energy[frame], sumvar);
*/
      done = (sumvar / nacoustic > maxdev);
    }
  }
  initframe = frame - 1;

/*
  for(quitframe = nframe - 1; energy[quitframe] < elimit; quitframe--);

  printf("%d\t%d\n", framestep * initframe, framestep * quitframe);
*/

  printf("%d\n", framestep * initframe);

  return(0);
}
