/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <sys/types.h>
# include <stdio.h>
# include <math.h>
# include "Slib.h"

/* Compute the Euclidean distance, or L2 norm, between two arrays */ 
float Sl2norm(float *vec0, float* vec1, int nitem) {
  float sum = 0.0;
  int i;

  for(i = 0; i < nitem; i++) {
    float diff = vec0[i] - vec1[i];
    sum += diff * diff;
  }

  return(sum);
}

/* The same as Sl2norm but take doubles as arguements */
double Sl2normd(double *vec0, double* vec1, int nitem) {
  double sum = 0.0;
  int i;

  for(i = 0; i < nitem; i++) {
    double diff = vec0[i] - vec1[i];
    sum += diff * diff;
  }

  return(sum);
}

/* compute the log likelihood of a vector assuming a Gaussian probability
   density function and a diagonal coovariance matrix */
float Sllikediag(float* vec, float* mean, float* var, int nitem) {
  int i;
  float nsuml = 0.0;

  for(i = 0; i < nitem; i++) {
    float dev = (vec[i] - mean[i]);
    nsuml += dev * dev / var[i] + log(2.0 * M_PI * var[i]);
  }
  return(-0.5 * nsuml);
}
