/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

/* 

Notes:

The maximum slope of 2 and minimum of 1/2 means that at least half the
local distance values are not used, and the number that are used goes to
zero as the ratio of word lengths goes to 2 or 1/2.

I wrote the code initally with a cylclic buffer of length 3 for
localDist and globalDist.  However, storing the back pointers is O(n^2),
so there seemed to be little point using 0(n) storage for the distances.
Also, my pointer shuffling code looked horrible and obscured the algorithm.

*/

# include <sys/types.h>
# include <stdio.h>
# include <math.h>
# include "Slib.h"

float Sdtw(float **unkTemp, int nunkTemp, float **refTemp, int nrefTemp,
	   int tempSize) {
  int i, j;
  float **localDist, **globalDist;
  float bestDist;

  localDist  = (float**) Smatrix(nunkTemp, nrefTemp, sizeof(**localDist));
  globalDist = (float**) Smatrix(nunkTemp, nrefTemp, sizeof(**globalDist));

  /* compute and store all the local distances */
  for(i = 0; i < nunkTemp; i++)
    for(j = 0; j < nrefTemp; j++)
      localDist[i][j] = Sl2norm(unkTemp[i], refTemp[j], tempSize);

  /* for the first frame the only possible match is at (0, 0) */
  globalDist[0][0] = localDist[0][0];
  for(j = 1; j < nrefTemp; j++)
    globalDist[0][j] = REALLY_VERY_BIG;

  /* in the second frame the only valid state is (1, 1) */
  globalDist[1][0] = REALLY_VERY_BIG;
  globalDist[1][1] = globalDist[0][0] + localDist[1][1];
  for(j = 2; j < nrefTemp; j++)
    globalDist[1][j] = REALLY_VERY_BIG;

  /* and do for the general case of the rest of the frames */
  for(i = 2; i < nunkTemp; i++) {
    
    /* */
    globalDist[i][0] = REALLY_VERY_BIG;
    globalDist[i][1] = globalDist[i - 1][0] + localDist[i][1];
  
    for(j = 2; j < nrefTemp; j++) {
      float topPath, midPath, botPath;

      if(globalDist[i-2][j-1] < 0.0)
	Spanic("globalDist[i-2][j-1] < 0.0\n: %d\t%d\t%f\n", i, j, globalDist[i-2][j-1]);

      topPath = globalDist[i-2][j-1] + localDist[i-1][j] + localDist[i][j];
      midPath = globalDist[i-1][j-1] + localDist[i][j];
      botPath = globalDist[i-1][j-2] + localDist[i][j-1] + localDist[i][j];

      /* find and store the smallest gloabal distance */
      if(topPath < midPath && topPath < botPath)
	globalDist[i][j] = topPath;
      else if (midPath < botPath)
	globalDist[i][j] = midPath;
      else
	globalDist[i][j] = botPath;
    }    
  }

  bestDist = globalDist[nunkTemp - 1][nrefTemp - 1];

  Sfree(localDist);
  Sfree(globalDist);

  return(bestDist);
}
