/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <stdio.h>
# include <sys/types.h>
# include <malloc.h>
# include "Slib.h"

char *Sfile2memory(char *filename) {
  FILE *stream = Sfopen(filename, "r");
  int	nbyte;
  char *data;

  Sfseek(stream, (long) 0, 2);
  nbyte = ftell(stream);
  data  = Smalloc(nbyte);
  Sfseek(stream, (long) 0, 0);
  Sfread(data, 1, nbyte, stream);
  Sfclose(stream);

  return(data);
}

void *Sfreadvector(FILE* stream, size_t* pnitem, size_t size) {
  int	nbyte;
  char *data;

  if(*pnitem == 0) {
    long startpos = ftell(stream);
    Sfseek(stream, (long) 0, 2);
    nbyte = ftell(stream) - startpos;
    Sfseek(stream, (long) 0, startpos);
    if(nbyte % size != 0)
      Spanic("Sfreadvector: non-integer number of records in file\n");
    *pnitem = nbyte / size;
  }
  else
    nbyte = *pnitem * size;

  data  = Smalloc(nbyte);
  Sfread(data, 1, nbyte, stream);
  Sfclose(stream);

  return((void*) data);
}

void *Sfreadmatrix(FILE* stream, size_t* pnitem0, size_t* pnitem1,size_t size){
  int	nbyte;
  char *data;

  if(*pnitem0 == 0) {
    long startpos = ftell(stream);
    Sfseek(stream, (long) 0, 2);
    nbyte = ftell(stream) - startpos;
    Sfseek(stream, (long) 0, startpos);
    if(nbyte % size != 0)
      Spanic("Sfreadvector: non-integer number of records in file\n");
    *pnitem0 = nbyte / size;
  }
  else
    nbyte = *pnitem0 * size;

  data  = Smalloc(nbyte);
  Sfread(data, 1, nbyte, stream);
  Sfclose(stream);

  return(data);
}


/* for compatability */
short *Sreadshortfile(char *filename, int headersize, int windowsize,
		     int *pnshort) {
  return(Sfreadwave(NULL, filename, headersize, windowsize, pnshort));
}

short *Sfreadwave(FILE *stream, char *filename, int headersize, int windowsize,
		  int *pnshort) {
  int	i, nbyte, nshort, hwindowsize = windowsize / 2;
  short *wave;

  if(filename == NULL) filename = "not known";
  if(stream == NULL) stream = Sfopen(filename, "r");
  Sfseek(stream, (long) 0, 2);
  nbyte = ftell(stream) - headersize;
  if(nbyte < 0)
    Spanic("Sfreadwave: header greater than total file: %s\n", filename);
  if(nbyte % 2 != 0)
    Spanic("Sfreadwave: extra bytes: %s\n", filename);
  nshort = nbyte / sizeof(short);
  if(nshort < windowsize)
    Spanic("Sfreadwave: less than one window of shorts: %s\n", filename);

  wave  = Svector(nshort + windowsize, sizeof(short));
  Sfseek(stream, (long) headersize, 0);
  Sfread(wave + hwindowsize, sizeof(short), nshort, stream);
  Sfclose(stream);

  /* duplicate ends */
  for(i = 1; i <= hwindowsize; i++)
    wave[hwindowsize - i] = wave[hwindowsize + i];
  for(i = 0; i < windowsize - hwindowsize; i++)
    wave[nshort + hwindowsize + i] = wave[nshort + hwindowsize - i - 2];

  *pnshort = nshort;
  return(wave);
}
