/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <sys/types.h>
# include <stdio.h>
# include <math.h>
# include "Slib.h"

/* This lot does non-linear frequency transformations */

/******************************************************************************
	author =	"R. A. W. Bladon and B. Lindblom",
	month =         "May",
	year =          "1981",
	journal =	"Journal of the Acoustical Society of America",
	title =         "Modeling the Judgement of Vowel Quality Differences",
	volume =	"69"

Original article:

	author=		"E. Zwicker and E. Terhart",
	title=		"Analytical expressions for critical-band rate and
			 critical bandwidth as a function of frequency",
	journal=	jasa,
	pages=		"1523-1525",
	volume=		"68(5)",
	year=		1980}

*/

double Bark_to_hz(double bark) {
    return(650.0 * sinh(bark / 7.0));
}

double Hz_to_bark(double hz) {
    double tmp = hz / 650.0;
    return(7.0 * log(tmp + sqrt(tmp * tmp + 1.0)));
}

/******************************************************************************

	author =	"Donald D. Greenwood",
	title=		"A Cochlear Frequency-Position Function for Several
	                 Species -- 29 Years Later",
	journal =	"Journal of the Acoustical Society of America",
	volume =	"87(6)",
	month =         jun,
	year =          1990,
*/

double Basilarmm2Hz(double basilarmm) {
  return(165.4 * (pow(10.0, 0.06 * basilarmm) - 1.0));
}

double Hz2Basilarmm(double hz) {
  return(log10(hz / 165.4 + 1.0) / 0.06);
}

/******************************************************************************
  Zwislocki(1965)
*/

double Mel2Hz(double mel) {
  return((exp(mel / 1125.0) - 1.0) / 0.0016);
}

double Hz2Mel(double hz) {
  return(1125.0 * log(0.0016 * hz + 1.0));
}

/******************************************************************************
  formula from JohnH taken from verion 4.0 of the auditory model
  latest erb function circa. 1989 from BBG & CJM
  derived from erb = 24.7 * ( 4.37e-3 * f +  1 )
*/

static double limit = 24.7  ; /* limit = k1 */
static double Q     = 9.265 ; /* Q = 1. / ( k1 * k2 ) */

double Hz2Erb(double frequency) {
  return ( limit + frequency / Q ) ;
}

double Hz2ErbScale(double frequency) {
  return(log(1.0 + frequency / Q / limit ) * Q );
}

double ErbScale2Hz(double E) {
  return((exp(E / Q) - 1.0) * Q * limit);
}

