/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SC_LICENSE for conditions on distribution and usage      *
*                                                                             *
******************************************************************************/

# include <stdio.h>
# include <errno.h>
# include <varargs.h>

/* The routine Spanic exits with a printf style error message */

void Spanic(va_alist)
va_dcl
{
  va_list args;
  char    *fmt;

  va_start(args);
  fmt = va_arg(args, char*);
  (void) vfprintf(stderr, fmt, args);
  va_end(args);
  exit(1);
}

/* The routine Sperror provdives a printf style error message then calls
   perror */

void Sperror(va_alist)
va_dcl
{
  va_list args;
  char    *fmt;

  va_start(args);
  fmt = va_arg(args, char*);
  (void) vfprintf(stderr, fmt, args);
  va_end(args);

  (void) fprintf(stderr, ": ");
  (void) fprintf(stderr, ": ");
  perror(NULL);
  exit(errno);
}

/* The remainder of these routines are replacements for the standard C
   library calls that perform error checking and exit if there are any
   problems */

# define SFOPEN_MAX_NTRY 4

/* Emulate fopen() but with added checking */

FILE *Sfopen(char *filename, char *mode) {
  FILE *stream;
  int ntry;

  /* Make ntry attempts to open a file.  This is useful for dodgy NFS
     links.  If you don't like this set SFOPEN_MAX_NTRY to 1. */
     
  for(ntry = 0; ntry < SFOPEN_MAX_NTRY &&
      (stream = fopen(filename, mode)) == NULL; ntry++)
    sleep(1);
  
  if(stream == NULL)
    Spanic("Sfopen(\"%s\", \"%s\") == NULL\n", filename, mode);
  return(stream);
}

void Sfclose(FILE *stream) {
  if((fclose(stream)) == EOF)
    Spanic("SC_fclose(%x) == EOF\n", (int) stream);
}

void Sfread(void *ptr, int size, int nitems, FILE *stream) {
  int nread;
  if((nread = fread(ptr, size, nitems, stream)) != nitems)
    Spanic("Sfread(%x, %d, %d, %x) == %d\n", (int) ptr, size, nitems, 
	   (int) stream, nread);
}

void Sfwrite(void *ptr, int size, int nitems, FILE *stream) {
  int nwrite;
  if((nwrite = fwrite(ptr, size, nitems, stream)) != nitems)
    Spanic("Sfwrite(%x, %d, %d, %x) == %d\n", (int) ptr, size, nitems, 
	   (int) stream, nwrite);
}

void Sfseek(FILE *stream, long offset, int ptrname) {
  if(fseek(stream, offset, ptrname) == -1)
    Sperror("Sfseek(%x, %d, %d)", (int) stream, offset, ptrname);
}
