/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <stdio.h>
# include <sys/types.h>
# include <malloc.h>
# include <string.h>
# include "Slib.h"

/* read characters from stream until a newline, '\n', or NULL, '\0', is
   encountered and return a pointer to the malloced storage with '\0'
   appended if it did not terminate the string.  On EOF, return NULL if
   nothing could be read, otherwise return the '\0' terminated string */

# define INITNLINE 256

char *line = NULL;
int  nline;

char *Sreadln(FILE *stream) {
  char *retline;
  int byte, nbyte = 0, done = 0;

  if(line == NULL) {
    nline = INITNLINE;
    line = Smalloc(nline);
  }

  while(!done) {
    if((byte = getc(stream)) == EOF) {
      line[nbyte++] = '\0';
    }
    else {
      if(nbyte == nline) {
	char *tmpline;
	
	nline += nline;
	tmpline = realloc(line, nline);
      
	if(tmpline == NULL)
	  Sperror("Sreadln: realloc(%d, %d): ", (unsigned int) line, nbyte);

	line = tmpline;
      }

      line[nbyte++] = byte;
    
      if(byte == '\n')
	line[nbyte++] = '\0';
    }
    
    if(line[nbyte - 1] == '\0')
      done = 1;
  }

  if(nbyte == 1 && byte == EOF)
    retline = NULL;
  else
    if((retline = strdup(line)) == NULL)
      Sperror("Sreadln: strdup(%d): ", (unsigned int) line);

  return(retline);
}

/* this is included in case you ever want to free the internal storage
   used by Sreadln() */

void Sreadln_reset() {
  line = NULL;
}
