/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <sys/types.h>
# include <stdio.h>
# include <math.h>
# include "Slib.h"

# define VERY_SMALL 1e-30

/* don't put out the 0th cepstral coefficient (1 to nceps inclusive) */
int spec2ceps(float *spec, int nspec, float *ceps, int nceps) {
  if(nceps != 0) {
    static int   nlndata = 0;
    static float *lndata = NULL;
    int i, j;

    if(nlndata != nspec) {
      if(nlndata != 0) Sfree((char*) lndata);
      lndata = (float*) Smalloc(nspec * sizeof(*lndata));
      nlndata = nspec;
    }

    for(i = 0; i < nspec; i++) lndata[i] = log(spec[i] + VERY_SMALL);

    for(i = 0; i < nceps; i++) {
      float sum = 0.0;

      for(j = 0; j < nspec; j++) 
        sum += lndata[j] * cos(M_PI * (j + 0.5) * (i + 1) / nspec);
      ceps[i] = sum;
    }
  }
  return(nceps);
}

void spec2ppos(float *spec, int nspec, float *ppos, int npeak) {
  int i, nfound = 0;

  for(i = 1; i < nspec - 1 && nfound < npeak; i++)
    if(spec[i] > spec[i - 1] && spec[i] >= spec[i + 1]) {
      float c = spec[i];
      float b = (spec[i + 1] - spec[i - 1]) / 2.0;
      float a = spec[i - 1] + b - c;
      float x = -b / (2.0 * a);

      ppos[nfound++] = i + x;
  }
  for( ; nfound < npeak; nfound++) ppos[nfound] = 0.0;
}

void spec2pamp(float *spec, int nspec, float *pamp, int npeak) {
  int i, nfound = 0;

  for(i = 1; i < nspec - 1 && nfound < npeak; i++)
    if(spec[i] > spec[i - 1] && spec[i] >= spec[i + 1]) {
      float c = spec[i];
      float b = (spec[i + 1] - spec[i - 1]) / 2.0;
      float a = spec[i - 1] + b - c;
      float x = -b / (2.0 * a);

      pamp[nfound++] = a * x * x + b * x + c;
  }
  for( ; nfound < npeak; nfound++) pamp[nfound] = 0.0;
}

#ifdef FIXED
void fft2chan(float *pspec, int size, float *bspec, int nchan, float minf, float maxf, float samplef) {
  float  minchan = Hz2Nonlinear(minf), maxchan = Hz2Nonlinear(maxf);
  float  chanstep = (maxchan - minchan) / (float) nchan;
  int    i, bin;

  for(bin = 0; bin < nchan; bin++) {
    int lo = size * Nonlinear2Hz((bin+0)*chanstep+minchan)/(samplef/2.0)+0.5;
    int hi = size * Nonlinear2Hz((bin+1)*chanstep+minchan)/(samplef/2.0)+0.5;

    bspec[bin] = 0.0;
    for(i = lo; i < hi; i++) bspec[bin] += pspec[LIMIT(i, size)];
  }
}
#endif
