/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <sys/types.h>
# include <stdio.h>
# include "Slib.h"

float Sviterbi(float **lpemit, float *lpinit, float **lptran, float *lpquit,
	       int nframe, int nstate, int *mlpath) {
  int **lastBestState = (int**) Smatrix(nframe,nstate,sizeof(**lastBestState));

  /* need a running sum of the least cost path to any given state at the
  last time frame and the current time frame */
  float *lastSum = Svector(nstate, sizeof(*lastSum));
  float *currSum = Svector(nstate, sizeof(*currSum));
  float leastSum;
  int   frame, i, j, bestPrev;

  /* set up the initial state probabilities */
  for(i = 0; i < nstate; i++)
    lastSum[i] = lpinit[i];

  /* for each time frame, find the best path to a given state */
  for(frame = 0; frame < nframe; frame++) {
    
    for(i = 0; i < nstate; i++) {
      leastSum = lastSum[0] + lptran[i][0];
      bestPrev = 0;
	    
      /* look over all last states and record the best path */
      for(j = 1; j < nstate; j++)
	if(lastSum[j] + lptran[i][j] < leastSum) {
	  leastSum = lastSum[j] + lptran[i][j];
	  bestPrev = j;
	}
      lastBestState[frame][i] = bestPrev;
      currSum[i] = leastSum + lpemit[frame][i];
    }
    for(i = 0; i < nstate; i++)
      lastSum[i] = currSum[i];
  }

  /* find the best final state */
  leastSum = lastSum[0] + lpquit[0];
  bestPrev = 0;

  for(i = 1; i < nstate; i++)
    if(lastSum[i] + lpquit[i] < leastSum) {
      bestPrev = i;
      leastSum = lastSum[i] + lpquit[i];
    }

      
  /* traceback from the best final state until the start of the observations */
  /* storing the best state sequence */
  if(mlpath != NULL) {
    mlpath[nframe - 1] = bestPrev;
    for(frame = nframe - 1; frame > 0; frame--) {
      bestPrev = lastBestState[frame][bestPrev];
      mlpath[frame - 1] = bestPrev;
    }
  }

  Sfree((char*) lastBestState);

  /* and return the probability of the best path */
  return(leastSum);
}
