/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage        *
*                                                                             *
******************************************************************************/

# include <math.h>
# include <stdio.h>
# include <sys/types.h>
# include "Slib.h"

/* set up a hamming window of nitem floating point numbers */

float *SmkHammwindow(int nitem) {
  float *window, k;
  int    i;

  k = 2.0 * M_PI / (nitem - 1);

  window = Svector(nitem, sizeof(*window));

  for(i = 0; i < nitem; i++)
    window[i] = 0.54 - 0.46 * cos(k * i);
  return(window);
}

/* Hamming window nitem shorts starting at wave and write to window */

void SHammwindow(short *wave, int nitem, float *window) {
  static int   nwindowf = 0;
  static float *windowf = NULL;
  int i;

  if(nwindowf != nitem) {
    if(nwindowf != 0) Sfree(windowf);
    windowf = SmkHammwindow(nitem);
    nwindowf = nitem;
  }

  for(i = 0; i < nitem; i++)
    window[i] = windowf[i] * wave[i];
}

/* Deduct mean then Hamming window nitem shorts starting at wave and 
   write to window */

void SzeroMeanHammWindow(short *wave, int nitem, float *window) {
  static int   nwindowf = 0;
  static float *windowf = NULL;
  int i, sum;
  float mean;

  if(nwindowf != nitem) {
    if(nwindowf != 0) Sfree(windowf);
    windowf = SmkHammwindow(nitem);
    nwindowf = nitem;
  }

  sum = 0;
  for(i = 0; i < nitem; i++)
    sum += wave[i];
  mean = (float) sum / nitem;

  for(i = 0; i < nitem; i++)
    window[i] = windowf[i] * (wave[i] - mean);
}
