/******************************************************************************
*                                                                             *
*       Copyright (C) 1993 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <stdio.h>
# include <sys/types.h>
# include <math.h>
# include <stdlib.h>
# include "Slib.h"

# define WINDOWSIZE  512
# define FRAMESIZE   256
# define NCEPS	      12
# define AIFFHEADERSIZE	86

typedef struct dist_type_struct {
  float	distance;
  int	index;
} dist_type;

static int dist_type_compare(dist_type *foo, dist_type *bar) {
  return(foo->distance > bar->distance ? 1 : -1);
}

int main(int argc, char **argv) {
  int  i, nfile, nref, *nframe;
  float *window, *acf, *lpc, *ref, *cep;
  float ***template;
  dist_type *globdist;

  /* n file is the number of files specified on the command line */
  nfile = argc - 1;
  
  if(nfile < 3)
    Spanic("not enough input files\n");

  /* nref in the number of reference files, the last one is "unknown" */
  nref = nfile - 1;

  window   = Svector(WINDOWSIZE, sizeof(*window));
  acf	   = Svector(NCEPS + 1, sizeof(*acf));
  lpc	   = Svector(NCEPS + 1, sizeof(*lpc));
  ref	   = Svector(NCEPS + 1, sizeof(*ref));
  cep	   = Svector(NCEPS + 1, sizeof(*cep));
  template = Svector(nfile, sizeof(*template));
  nframe   = Svector(nfile, sizeof(*nframe));
  globdist = Svector(nref,  sizeof(*globdist));

  /* calculate a template of cepstral coefficients per file */
  for(i = 0; i < nfile; i++) {
    short *sample;
    int   nsample, j;

    sample = Sreadshortfile(argv[i + 1],  AIFFHEADERSIZE, WINDOWSIZE,&nsample);
    nframe[i] = nsample / FRAMESIZE;
    template[i] = (float**) Smatrix(nframe[i], NCEPS, sizeof(***template));

    for(j = 0; j < nframe[i]; j++) {
      int k;

      SHammwindow(sample + j * FRAMESIZE, WINDOWSIZE, window);
      Swav2acf(window, WINDOWSIZE, acf, NCEPS + 1);
      Sacf2lpc(acf, NCEPS, lpc, ref);
      Slpc2cep(lpc, NCEPS, cep, NCEPS);
      for(k = 0; k < NCEPS; k++)
	template[i][j][k] = cep[k + 1];
    }
  }

  /* Match the i'th reference against the unknown using DTW */
  for(i = 0; i < nref; i++) {
    globdist[i].distance = Sdtw(template[nref], nframe[nref], template[i],
				nframe[i], NCEPS);
    globdist[i].index = i;
  }

  /* sort the global distances putting the lowest score first */
  qsort(globdist, nref, sizeof(*globdist), dist_type_compare);

  /* and print out the result, with the best matching filename first */
  for(i  = 0; i < nref; i++)
    printf("%f\t%s\n", globdist[i].distance, argv[1 + globdist[i].index]);

  /* tidy up by freeing memory */
  Sfree(acf);
  Sfree(lpc);
  Sfree(ref);
  for(i  = 0; i < nfile; i++) Sfree(template[i]);
  Sfree(template);
  
  return(0);
}
