/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <sys/types.h>
# include <stdio.h>
# include <math.h>
# include "Slib.h"

# define DEFAULT_HEADERSIZE	0
# define DEFAULT_FRAMESTEP	200
# define DEFAULT_WINDOWSIZE	(2 * DEFAULT_FRAMESTEP)
# define DEFAULT_LPORDER	16

int main(argc, argv) int argc; char **argv; {
  int swapbytes;
  int headersize   = DEFAULT_HEADERSIZE;
  int framestep    = DEFAULT_FRAMESTEP;
  int windowsize   = DEFAULT_WINDOWSIZE;
  int lporder	   = DEFAULT_LPORDER;
  FILE *filei = NULL, *fileo = NULL;
  short *wave;
  float *wwave, *acf, *lpc, *ref;
  int   nsample, frame, nframe;

  Sgetargs(argc, argv, "-h", "-help", NULL,
	"-swapbytes",	NULL, &swapbytes, "swap the byte order of the signals",
	"-headersize",	NULL, &headersize, "size of header in bytes",
	"-framestep",	"%d", &framestep, "analysis frame step",
	"-windowsize",	"%d", &windowsize, "analysis window size",
	"-lporder",	"%d", &lporder, "order of LP analysis",
	NULL,
	"input", "r", &filei, NULL, "original speech file",
	"ouptut", "w", &fileo, NULL, "LPC vocoded file",
	NULL);

  wwave = Svector(windowsize, sizeof(*wwave));

  /* grab space for the autocorrelation functions */
  acf = Svector(lporder + 1, sizeof(*acf));
  lpc = Svector(lporder + 1, sizeof(*lpc));
  ref = Svector(lporder + 1, sizeof(*ref));

  /* read in the waveform */
  wave = Sfreadwave(filei, NULL, headersize, windowsize - framestep, &nsample);
  if(swapbytes)
    Sswab(wave, wave, 2 * (nsample + windowsize - framestep));

  /* compute total number of frames */
  nframe = nsample / framestep;

  for(frame = 0; frame < nframe; frame++) {
    float pred_res;
    SzeroMeanHammWindow(wave + frame * framestep, windowsize, wwave);
    Swav2acf(wwave, windowsize, acf, lporder + 1);
    pred_res = Sacf2lpc(acf, lporder, lpc, ref);

    printf("%d\t%f\t%f\n", frame, acf[0], pred_res);
  }

  return(0);
}
