/******************************************************************************
*                                                                             *
*       Copyright (C) 1993,1994 Tony Robinson				      *
*                                                                             *
*       See the file SLICENSE for conditions on distribution and usage	      *
*                                                                             *
******************************************************************************/

# include <sys/types.h>
# include <stdio.h>
# include <math.h>
# include "Slib.h"

# define DEFAULT_HEADERSIZE	0
# define DEFAULT_FRAMESTEP	200
# define DEFAULT_WINDOWSIZE	(2 * DEFAULT_FRAMESTEP)
# define DEFAULT_MAX_PERIOD	DEFAULT_FRAMESTEP

char *progname = "ptrack";

int main(argc, argv) int argc; char **argv; {
  int headersize   = DEFAULT_HEADERSIZE;
  int maxperiod    = DEFAULT_MAX_PERIOD;
  int framestep    = DEFAULT_FRAMESTEP;
  int windowsize   = DEFAULT_WINDOWSIZE;
  int swapbytes;
  FILE *filei;
  FILE *fileo = stdout;
  float	*wwave, **acf;
  short *wave;
  int   i, j, nsample, frame, nframe;

  Sgetargs(argc, argv, "-h", "-help", NULL,
	"-swapbytes", NULL, &swapbytes, "change the byte order of the input",
	"-headersize",NULL, &headersize, "size of header in bytes",
	"-maxperiod", "%d", &maxperiod, "maximum pitch period",
	"-framestep", "%d", &framestep, "analysis frame step",
	"-windowsize","%d", &windowsize, "analysis window size",
	NULL,
	"input", "r", &filei, NULL, "original speech file",
	"output", "w", &fileo, NULL, "file containing pitch contour",
	NULL);

  /* set up the space to store a windowed frame */
  wwave = Svector(windowsize, sizeof(*wwave));

  /* read in the waveform */
  wave = Sfreadwave(filei, NULL, headersize, windowsize - framestep, &nsample);

  /* grab space for the autocorrelation functions */
  nframe = nsample / framestep;
  acf = (float**) Smatrix(nframe, maxperiod, sizeof(**acf));

  /* window then autocorelate the speech */
  for(frame = 0; frame < nframe; frame++) {
    SzeroMeanHammWindow(wave + frame * framestep, windowsize, wwave);
    /* this should be replaced with a call the Swav2fftacf */
    Swav2acf(wwave, windowsize, acf[frame], maxperiod);
  }

  /* free the space for the waveform */
  Sfree(wave);
  
  /* Dynamic programming starts here */
#define FRAMEVAR (1.0 / 128.0)
  {
    float  *lpinit = Svector(maxperiod, sizeof(*lpinit));
    float  *lpquit = Svector(maxperiod, sizeof(*lpquit));
    float **lptran = (float**) Smatrix(maxperiod, maxperiod, sizeof(**lptran));
    float **lpemit = (float**) Smatrix(nframe, maxperiod, sizeof(**lpemit));
    int    *mlpath = Svector(nframe, sizeof(*mlpath));
    float logzero = -1e30;

    /* set up the transition probabilities */
    for(j = 0; j < maxperiod; j++)
      lptran[0][j] = - logzero;
    for(i = 1; i < maxperiod; i++)
      for(j = 0; j < maxperiod; j++)
	lptran[i][j] = (i - j) * (i - j) / (2.0 * i * i * FRAMEVAR);

    /* set the probability of starting or ending in state zero to zero */
    lpinit[0] = lpquit[0] = - logzero;

    /* set up the other initial and final state probabilties to be
       the global average over all the utterance */
    { float gacf0 = 0.0;

      for(frame = 0; frame < nframe; frame++) gacf0 += acf[frame][0];

      for(i = 1; i < maxperiod; i++) {
	float gacf = 0.0;

	for(frame = 0; frame < nframe; frame++) gacf += acf[frame][i];

	lpinit[i] = lpquit[i] = 1.0 - gacf / gacf0;
      }
    }

    /* set up the emission probabilities */
    for(frame = 0; frame < nframe; frame++) {
      lpemit[frame][0] = - logzero;
      for(i = 1; i < maxperiod && acf[frame][i] > 0.5; i++)
	lpemit[frame][i] = - logzero;      
      for( ; i < maxperiod; i++)
	lpemit[frame][i] = 1.0 - acf[frame][i] / acf[frame][0];
    }

    /* begin viterbi */
    (void) Sviterbi(lpemit, lpinit, lptran, lpquit, nframe, maxperiod, mlpath);

    /* add the trace to the acf */
    for(frame = 0; frame < nframe; frame++)
      acf[frame][mlpath[frame]] = - acf[frame][0];

    Sfree((char*) lpinit);
    Sfree((char*) lpquit);
    Sfree((char*) lptran);
    Sfree((char*) lpemit);
    Sfree((char*) mlpath);
  }

  /* print out */
  fprintf(stdout, "P2\n");
  fprintf(stdout, "%d %d\n", nframe, maxperiod);
  fprintf(stdout, "255\n");
  for(i = maxperiod - 1; i >= 0; i--)
    for(frame = 0; frame < nframe; frame++)
      fprintf(stdout, "%d\n", 128 + (int) floor(127.99 * acf[frame][i] / acf[frame][0]));
  
  return(0);
}
